<?php

namespace Advanced_Sidebar_Menu\Widget;

use Advanced_Sidebar_Menu\Menus\Menu_Abstract;
use Advanced_Sidebar_Menu\Menus\Navigation;

/**
 * Navigation Menu Widget
 *
 * @author OnPoint Plugins
 * @since  3.3.0
 */
class Navigation_Menu extends Widget_Abstract {
	const NAME                = 'advanced-sidebar-navigation-menu';
	const TITLE               = Menu_Abstract::TITLE;
	const NAVIGATION_MENU     = 'navigation_menu';
	const CURRENT_PARENT_ONLY = 'current-parent-only';
	const TOP_LEVEL_SIBLINGS  = 'top-level-siblings';
	const DISPLAY_NO_CURRENT  = 'display-no-current';

	/**
	 * Defaults for all navigation menu widgets.
	 *
	 * @var array
	 */
	protected static $defaults = [
		self::TITLE                   => false,
		self::NAVIGATION_MENU         => false,
		self::CURRENT_PARENT_ONLY     => false,
		self::TOP_LEVEL_SIBLINGS      => false,
		self::DISPLAY_NO_CURRENT      => false,
		// Must be checked to allow accordion and block styles to work properly.
		Menu_Abstract::INCLUDE_PARENT => 'checked',
	];


	/**
	 * Construct the widget.
	 */
	public function __construct() {
		$widget_ops = [
			'classname'   => 'advanced-sidebar-menu advanced-sidebar-navigation',
			'description' => __( 'Creates a sidebar menu from a Navigation Menu in Appearance -> Menus', 'advanced-sidebar-menu' ),
		];
		$control_ops = [
			'width' => wp_is_mobile() ? false : 620,
		];

		parent::__construct( self::NAME, __( 'Advanced Sidebar Navigation Menu', 'advanced-sidebar-menu' ), $widget_ops, $control_ops );

		$this->hook();
	}


	/**
	 * Add widget sections via actions.
	 *
	 * @notice Anything using the column actions must use the $widget class passed
	 *         via do_action instead of $this
	 *
	 * @return void
	 */
	protected function hook() {
		static $hooked;
		if ( null !== $hooked ) {
			return;
		}
		$hooked = true;

		add_action( 'advanced-sidebar-menu/widget/navigation-menu/left-column', [ $this, 'box_menu' ], 5, 2 );
		add_action( 'advanced-sidebar-menu/widget/navigation-menu/right-column', [
			$this,
			'box_display_parent_only',
		], 10, 2 );
	}


	/**
	 * Save widget settings.
	 *
	 * @param array $new_instance - New widget settings.
	 * @param array $old_instance - Old widget settings.
	 *
	 * @return array|mixed
	 */
	public function update( $new_instance, $old_instance ) {
		return apply_filters( 'advanced-sidebar-menu/widget/navigation-menu/update', $new_instance, $old_instance );
	}


	/**
	 * Render the select menu settings box.
	 *
	 * @param array           $instance - Widget settings.
	 * @param Widget_Abstract $widget   - Widget class.
	 *
	 * @return void
	 */
	public function box_menu( array $instance, $widget ) {
		$menus = wp_get_nav_menus();
		$instance = $widget->set_instance( $instance, self::$defaults );
		$nav_menu = isset( $instance[ self::NAVIGATION_MENU ] ) ? $instance[ self::NAVIGATION_MENU ] : '';

		?>
		<div class="advanced-sidebar-menu-column-box">
			<p>
				<label for="<?php echo esc_attr( $widget->get_field_id( self::NAVIGATION_MENU ) ); ?>">
					<?php esc_html_e( 'Select Menu:', 'advanced-sidebar-menu' ); ?>
				</label>
				<select
					id="<?php echo esc_attr( $widget->get_field_id( self::NAVIGATION_MENU ) ); ?>"
					name="<?php echo esc_attr( $widget->get_field_name( self::NAVIGATION_MENU ) ); ?>">
					<option value="0">
						<?php esc_html_e( '&mdash; Select &mdash;', 'advanced-sidebar-menu' ); ?>
					</option>
					<?php
					foreach ( $menus as $menu ) {
						?>
						<option
							value="<?php echo esc_attr( $menu->term_id ); ?>" <?php selected( $nav_menu, $menu->term_id ); ?>>
							<?php echo esc_html( $menu->name ); ?>
						</option>
						<?php
					}
					?>
				</select>
			</p>
		</div>
		<?php
	}


	/**
	 * Display the "Display the current item's section only." options.
	 *
	 * @param array           $instance - Widget settings.
	 * @param Widget_Abstract $widget   - Widget arguments.
	 *
	 * @return void
	 */
	public function box_display_parent_only( array $instance, $widget ) {
		?>
		<div class="advanced-sidebar-menu-column-box">
			<p>
				<?php $widget->checkbox( self::CURRENT_PARENT_ONLY, 'siblings-no-children' ); ?>
				<label><?php esc_html_e( "Display the current item's section only.", 'advanced-sidebar-menu' ); ?>
				</label>
				<br />
				<span class="description">
					<em>
						<?php esc_html_e( 'Only display the current item and any parents, grandparents, siblings, or children of the current item.', 'advanced-sidebar-menu' ); ?>
					</em>
				</span>
			</p>
			<div <?php $widget->hide_element( self::CURRENT_PARENT_ONLY, 'siblings-no-children' ); ?>>

				<p>
					<?php $widget->checkbox( self::TOP_LEVEL_SIBLINGS, 'siblings-wrap' ); ?>
					<label>
						<?php esc_html_e( 'Display top-level siblings.', 'advanced-sidebar-menu' ); ?>
					</label>
					<br />
					<span class="description">
				<em><?php esc_html_e( 'Display top-level menu items when on a top-level menu item that has no children.', 'advanced-sidebar-menu' ); ?></em>
			</span>
				</p>

				<p>
					<?php $widget->checkbox( self::DISPLAY_NO_CURRENT ); ?>
					<label>
						<?php esc_html_e( 'Display top-level items when not viewing a menu item', 'advanced-sidebar-menu' ); ?>
					</label>
					<br />
					<span class="description">
				<em><?php esc_html_e( 'Display the top-level menu items when viewing a page that is not part of the menu.', 'advanced-sidebar-menu' ); ?></em>
			</span>
				</p>
			</div>
		</div>
		<?php
	}


	/**
	 * Widget form.
	 *
	 * @param array $instance - Widget settings.
	 *
	 * @return void
	 */
	public function form( $instance ) {
		$instance = $this->set_instance( $instance, self::$defaults );
		do_action( 'advanced-sidebar-menu/widget/navigation-menu/before-form', $instance, $this );
		?>
		<p xmlns="http://www.w3.org/1999/html">
			<label for="<?php echo esc_attr( $this->get_field_id( self::TITLE ) ); ?>">
				<?php esc_html_e( 'Title', 'advanced-sidebar-menu' ); ?>:
			</label>
			<input
				id="<?php echo esc_attr( $this->get_field_id( self::TITLE ) ); ?>"
				name="<?php echo esc_attr( $this->get_field_name( self::TITLE ) ); ?>"
				class="widefat"
				type="text"
				value="<?php echo esc_attr( $instance[ self::TITLE ] ); ?>" />
		</p>
		<div class="advanced-sidebar-menu-column">
			<?php
			do_action( 'advanced-sidebar-menu/widget/navigation-menu/left-column', $instance, $this );
			?>
		</div>
		<div class="advanced-sidebar-menu-column advanced-sidebar-menu-column-right">
			<?php
			do_action( 'advanced-sidebar-menu/widget/navigation-menu/right-column', $instance, $this );
			?>
		</div>
		<div class="advanced-sidebar-menu-full-width"><!-- clear --></div>
		<?php
		do_action( 'advanced-sidebar-menu/widget/navigation-menu/after-form', $instance, $this );
	}


	/**
	 * Widget Output
	 *
	 * @param array $args     - Widget registration args.
	 * @param array $instance - Widget settings.
	 *
	 * @return void
	 */
	public function widget( $args, $instance ) {
		$instance = (array) wp_parse_args( $instance, self::$defaults );
		$asm = Navigation::factory( $instance, $args );

		do_action( 'advanced-sidebar-menu/widget/before-render', $asm, $this );

		$asm->render();

		do_action( 'advanced-sidebar-menu/widget/after-render', $asm, $this );
	}

}
