<?php

namespace Advanced_Sidebar_Menu\Widget_Options\Category;

use Advanced_Sidebar_Menu\Traits\Singleton;
use Advanced_Sidebar_Menu\Widget\Widget_Abstract;
use Advanced_Sidebar_Menu\Widget_Options\Widget_Options_Abstract;

/**
 * Order by widget options.
 */
class Order_By extends Widget_Options_Abstract {
	use Singleton;

	const NAME = true;

	const ORDER    = 'order';
	const ORDER_BY = 'order_by';

	const ORDER_BY_NAME  = 'name';
	const ORDER_BY_COUNT = 'count';
	const ORDER_BY_SLUG  = 'slug';

	const ORDER_ASC  = 'ASC';
	const ORDER_DESC = 'DESC';


	/**
	 * Actions and filters.
	 */
	protected function hook() {
		add_filter( 'advanced-sidebar-menu/menus/category/order-by', [ $this, 'get_orderby' ], 10, 3 );
		add_filter( 'advanced-sidebar-menu/menus/category/order', [ $this, 'get_order' ], 10, 3 );

		add_action( 'advanced-sidebar-menu/widget/category/right-column', [ $this, 'render_fields' ], 15, 2 );
	}


	/**
	 * Filter the menu orderby using the widget setting.
	 *
	 * @param string $orderby  - Current orderby (default to 'name').
	 * @param array  $args     - Widget registration args.
	 * @param array  $instance - Widget settings.
	 *
	 * @return string
	 */
	public function get_orderby( $orderby, array $args, array $instance ) {
		if ( isset( $instance[ self::ORDER_BY ] ) ) {
			return $instance[ self::ORDER_BY ];
		}

		return $orderby;
	}


	/**
	 * Filter the menu order by using the widget setting.
	 *
	 * @param string $order    - Current orderby (default to "ASC").
	 * @param array  $args     - Widget registration args.
	 * @param array  $instance - Widget settings.
	 *
	 * @return string
	 */
	public function get_order( $order, array $args, array $instance ) {
		if ( isset( $instance[ self::ORDER ] ) ) {
			return $instance[ self::ORDER ];
		}

		return $order;
	}


	/**
	 * Render the widget fields.
	 *
	 * @param array           $instance - Widget settings.
	 * @param Widget_Abstract $widget   - Widget class.
	 *
	 * @return void
	 */
	public function render_fields( array $instance, $widget ) {
		$instance[ self::ORDER ] = empty( $instance[ self::ORDER ] ) ? 'ASC' : $instance[ self::ORDER ];
		$instance[ self::ORDER_BY ] = empty( $instance[ self::ORDER_BY ] ) ? '' : $instance[ self::ORDER_BY ];
		?>
		<div class="advanced-sidebar-menu-column-box">
			<p>
				<label for="<?php echo esc_attr( $widget->get_field_id( self::ORDER_BY ) ); ?>">
					<?php esc_html_e( 'Order By', 'advanced-sidebar-menu' ); ?>:
				</label>
				<select
					id="<?php echo esc_attr( $widget->get_field_id( self::ORDER_BY ) ); ?>"
					name="<?php echo esc_attr( $widget->get_field_name( self::ORDER_BY ) ); ?>">
					<option value="">
						<?php esc_html_e( 'Created', 'advanced-sidebar-menu' ); ?>
					</option>
					<?php
					$order_by = [
						self::ORDER_BY_NAME  => __( 'Name', 'advanced-sidebar-menu' ),
						self::ORDER_BY_COUNT => __( 'Count', 'advanced-sidebar-menu' ),
						self::ORDER_BY_SLUG  => __( 'Slug', 'advanced-sidebar-menu' ),
					];

					foreach ( $order_by as $key => $order ) {
						printf( '<option value="%s" %s>%s</option>', esc_attr( $key ), selected( $instance[ self::ORDER_BY ], $key, false ), esc_html( $order ) );
					}
					?>
				</select>
			</p>
			<p>
				<label for="<?php echo esc_attr( $widget->get_field_id( self::ORDER ) ); ?>">
					<?php esc_html_e( 'Order', 'advanced-sidebar-menu' ); ?>:
				</label>
				<select
					id="<?php echo esc_attr( $widget->get_field_id( self::ORDER ) ); ?>"
					name="<?php echo esc_attr( $widget->get_field_name( self::ORDER ) ); ?>">
					<?php
					$order = [
						self::ORDER_ASC  => __( 'Ascending', 'advanced-sidebar-menu' ),
						self::ORDER_DESC => __( 'Descending', 'advanced-sidebar-menu' ),
					];

					foreach ( $order as $key => $_order ) {
						printf( '<option value="%s" %s>%s</option>', esc_attr( $key ), selected( $instance[ self::ORDER ], $key, false ), esc_html( $_order ) );
					}
					?>
				</select>
			</p>
		</div>
		<?php
	}

}
