<?php

namespace Advanced_Sidebar_Menu\Widget_Options\Shared;

use Advanced_Sidebar_Menu\Menus\Menu_Abstract;
use Advanced_Sidebar_Menu\Traits\Singleton;
use Advanced_Sidebar_Menu\Widget\Widget_Abstract;
use Advanced_Sidebar_Menu\Widget_Options\Widget_Options_Abstract;

/**
 * Style targeting widget settings.
 *
 * @author OnPoint Plugins
 * @since  3.2.0
 */
class Style_Targeting extends Widget_Options_Abstract {
	use Singleton;

	const APPLY_CURRENT_STYLES = 'apply_current_page_styles_to_parent';
	const APPLY_PARENT_STYLES  = 'apply_current_page_parent_styles_to_parent';

	const BORDER_COLOR = 'border_color';
	const BLOCK_STYLE  = 'block_style';
	const BORDER       = 'border';
	const BULLET_STYLE = 'bullet_style';

	/**
	 * Default widget settings.
	 *
	 * @var array
	 */
	private $defaults = [
		self::BLOCK_STYLE  => false,
		self::BORDER       => false,
		self::BULLET_STYLE => false,
		self::BORDER_COLOR => false,
	];


	/**
	 * Actions.
	 */
	protected function hook() {
		add_action( 'advanced-sidebar-menu/widget/page/left-column', [ $this, 'render_box' ], 21, 2 );
		add_action( 'advanced-sidebar-menu/widget/category/left-column', [ $this, 'render_box' ], 21, 2 );
		add_action( 'advanced-sidebar-menu/widget/navigation-menu/left-column', [ $this, 'render_box' ], 21, 2 );
	}


	/**
	 * Render CSS for border based on selected options.
	 *
	 * @param string $widget_selector - CSS target id.
	 * @param array  $options         - Widget settings.
	 *
	 * @return string
	 */
	public function get_border_styles( $widget_selector, array $options ) {
		$color = empty( $options[ self::BORDER_COLOR ] ) ? 'black' : $options[ self::BORDER_COLOR ];
		$out = '';

		if ( ! empty( $options[ Menu_Abstract::INCLUDE_PARENT ] ) ) {
			$out .= $widget_selector . '> ul{
			border: 1px solid ' . $color . ' !important;
			padding: 0 !important;
			}';
		} else {
			$out .= $widget_selector . '> ul{
			border-left: 1px solid ' . $color . ' !important;
			border-right: 1px solid ' . $color . ' !important;
			border-bottom: 1px solid ' . $color . ' !important;
			padding: 0 !important;
			}';
		}
		if ( ! empty( $options[ Menu_Abstract::INCLUDE_CHILDLESS_PARENT ] ) ) {
			// because we don't want to risk doubling up borders if only the top parent is shown.
			$out .= $widget_selector . ' .child-sidebar-menu > li > a{border-top: 1px solid ' . $color . ' !important;}';
		} else {
			$out .= $widget_selector . ' .parent-sidebar-menu > li > a{border-bottom: 1px solid ' . $color . ' !important;}';
		}

		return $out;
	}


	/**
	 * If the widget has block styles turned on we generate them here.
	 *
	 * @param string        $css_id  - Id of the widget wrap.
	 * @param array         $options - Widget options.
	 * @param Menu_Abstract $menu    - Current menu.
	 *
	 * @since 3.8.6 - Support filtering of number of levels of styles to support.
	 *
	 * @return string
	 */
	public function get_block_styles( $css_id, $options, Menu_Abstract $menu ) {
		// make room for icons.
		if ( $menu->checked( Accordion::ENABLE_ACCORDION ) ) {
			$out = $css_id . 'a{display:block !important; padding: 5px 25px 5px 10px !important;}';
		} else {
			$out = $css_id . 'a{display:block !important; padding: 5px 10px !important;}';
		}

		// ul ol.
		$out .= $css_id . 'ul,' . $css_id . 'ol{margin: 0 0 1.75em !important; padding-left: 0 !important;}';
		$out .= $css_id . 'ul ul,' . $css_id . 'ol ol{margin: 0 0 0 !important; padding: 0 !important;}';
		$out .= $css_id . 'li{margin: 0 !important; padding: 0 !important}';

		// li.
		$levels = apply_filters( 'advanced-sidebar-menu-pro/styles/block-style/levels', min( 10, $menu->get_levels_to_display() ), $options );
		for ( $i = 2; $i <= $levels; $i ++ ) {
			$out .= $css_id . str_repeat( 'li ', $i ) . "a{padding-left: {$i}0px !important;}";
		}

		// icons.
		$out .= $css_id . 'li{position:relative !important}';
		$out .= $css_id . '.dashicons{position:absolute !important; top: 5px !important; right: 5px !important;}';

		return $out;
	}


	/**
	 * List of available bullet types.
	 *
	 * @return array
	 */
	public function get_bullet_types() {
		$types = [
			'circle'      => __( 'Circle', 'advanced-sidebar-menu' ),
			'decimal'     => __( 'Decimal', 'advanced-sidebar-menu' ),
			'disc'        => __( 'Disc', 'advanced-sidebar-menu' ),
			'square'      => __( 'Square', 'advanced-sidebar-menu' ),
			'upper-alpha' => __( 'Uppercase Alphabet', 'advanced-sidebar-menu' ),
			'lower-alpha' => __( 'Lowercase Alphabet', 'advanced-sidebar-menu' ),
			'none'        => __( 'None', 'advanced-sidebar-menu' ),
		];

		return apply_filters( 'advanced-sidebar-menu/styles/bullet_types', $types );
	}


	/**
	 * Add the styles box to the widget options.
	 *
	 * The basic version no longer has a style box, so we
	 * wrap our call to `render_fields` with our own box here.
	 *
	 * @param array           $instance - Widget settings.
	 * @param Widget_Abstract $widget   - Widget class.
	 */
	public function render_box( array $instance, $widget ) {
		?>
		<div class="advanced-sidebar-menu-column-box">
			<?php
			$this->render_fields( $instance, $widget );
			do_action( 'advanced-sidebar-menu/widget-options/shared/shown-styles/box', $instance, $widget );
			?>
		</div>
		<?php
	}


	/**
	 * Render the widget fields.
	 *
	 * @param array           $instance - Widget settings.
	 * @param Widget_Abstract $widget   - Widget class.
	 *
	 * @notice Front end rendering is handled by the Styles class
	 *
	 * @return void
	 */
	public function render_fields( array $instance, $widget ) {
		$instance = (array) wp_parse_args( $instance, $this->defaults );

		?>
		<p>
			<?php $widget->checkbox( self::APPLY_CURRENT_STYLES ); ?>
			<label>
				<?php esc_html_e( 'Apply current item styles to the highest level parent page (when applicable)', 'advanced-sidebar-menu' ); ?>
			</label>

		</p>
		<p>
			<?php $widget->checkbox( self::APPLY_PARENT_STYLES ); ?>
			<label>
				<?php esc_html_e( 'Apply current item parent styles to the highest level parent page (when applicable)', 'advanced-sidebar-menu' ); ?>
			</label>
		</p>

		<p>
			<?php $widget->checkbox( self::BLOCK_STYLE, 'border-wrap' ); ?>
			<label><?php esc_html_e( 'Use block styling', 'advanced-sidebar-menu' ); ?></label>
		</p>
		<div <?php $widget->hide_element( self::BLOCK_STYLE, 'border-wrap' ); ?>>
			<p>
				<?php $widget->checkbox( self::BORDER, 'border-color-wrap' ); ?>
				<label><?php esc_html_e( 'Show borders', 'advanced-sidebar-menu' ); ?></label>
			</p>

			<div <?php $widget->hide_element( self::BORDER, 'border-color-wrap' ); ?>
				class="advanced-sidebar-menu-border-color">
				<p>
					<label>
						<?php esc_html_e( 'Border color', 'advanced-sidebar-menu' ); ?>:
					</label>
					<?php
					printf( '<input type="text" name="%s" value="%s" id="%s" class="asm-color-select">',
						esc_attr( $widget->get_field_name( self::BORDER_COLOR ) ),
						esc_html( $instance[ self::BORDER_COLOR ] ),
						esc_attr( $widget->get_field_id( self::BORDER_COLOR ) ) );
					?>
				</p>
			</div>
		</div>
		<p>
			<label
				for="<?php echo esc_attr( $widget->get_field_id( self::BULLET_STYLE ) ); ?>">
				<?php esc_html_e( 'Bullet style', 'advanced-sidebar-menu' ); ?>:
			</label>
			<select
				name="<?php echo esc_attr( $widget->get_field_name( self::BULLET_STYLE ) ); ?>"
				id="<?php echo esc_attr( $widget->get_field_id( self::BULLET_STYLE ) ); ?>">
				<option value="">
					<?php esc_html_e( '- default -', 'advanced-sidebar-menu' ); ?>
				</option>
				<?php
				foreach ( $this->get_bullet_types() as $_type => $__label ) {
					printf( '<option value="%s" %s>%s</option>', esc_attr( $_type ), selected( $instance[ self::BULLET_STYLE ], esc_attr( $_type ) ), esc_html( $__label ) );
				}
				?>
			</select>
		</p>
		<?php
	}

}
